import os
from java.lang import System
from java.lang import Exception as JavaException

SCRIPT_VERSION='12/04/2010'
alias = '\"ibm websphere datapower xc10\"'
saveNeeded = 0
errors = 0

def convertToList(inlist):
     outlist=[]
     if (len(inlist)>0 and inlist[0]=='[' and inlist[len(inlist)-1]==']'):
        inlist = inlist[1:len(inlist)-1]
        clist = inlist.split("\"")
     else:
        clist = inlist.split("\n")
     for elem in clist:
        elem=elem.rstrip();
        if (len(elem)>0):
           outlist.append(elem)
     return outlist

def addXC10PublicCert(printPrefix, pathToCert, cellName, nodeName, force):
#     print 'addXC10PublicCert('+pathToCert+','+cellName+','+nodeName+','+str(force)+')'
     keyStoreScope = '(cell):'+cellName
     keyStoreName = 'CellDefaultTrustStore'
     if (len(nodeName)>0):
          keyStoreName = 'NodeDefaultTrustStore'
          keyStoreScope = keyStoreScope+':(node):'+nodeName
        
     # Check if the alias is present already - if so, skip unless force is true
     try:
          AdminTask.getSignerCertificate('[-keyStoreName '+keyStoreName+' -certificateAlias '+alias+' -keyStoreScope '+keyStoreScope+']')
          if (force):
               print printPrefix+'XC10 public certificate found in '+keyStoreName+' '+keyStoreScope+' forcing removal ...'
               success = removeXC10PublicCert(printPrefix, cellName, nodeName)
               if (success > -1):
                    AdminTask.addSignerCertificate('[-certificateAlias '+alias+' -certificateFilePath '+pathToCert+' -keyStoreScope '+keyStoreScope+' -keyStoreName '+keyStoreName+' ]')
                    print printPrefix+'Added XC10 public certificate into '+keyStoreName+' '+keyStoreScope
                    global saveNeeded 
                    saveNeeded = 1
               else:
                    print printPrefix+'FAILURE: Could not remove the XC10 public certificate in keystore '+keyStoreName+' '+keyStoreScope+'.'
                    print printPrefix+'         Try to manually remove certificate then re-run: AdminTask.deleteSignerCertificate(\'[-keyStoreName '+keyStoreName+' -certificateAlias '+alias+' -keyStoreScope '+keyStoreScope+']\')'
                    global errors
                    errors = errors + 1
                    return
          else:
               print printPrefix+'XC10 public certificate found in '+keyStoreName+' '+keyStoreScope+' will not re-add.'
               return
     except JavaException, e:
          msg = str(e)
          msg.rstrip()
          msg =  msg[0:len(msg)-1]
          # Exception is thrown if the certificate is not present
          if (msg.find('does not exist') > -1):
               # certificate not already in truststore, so add it
               try:
                    AdminTask.addSignerCertificate('[-certificateAlias '+alias+' -certificateFilePath '+pathToCert+' -keyStoreScope '+keyStoreScope+' -keyStoreName '+keyStoreName+' ]')
                    global saveNeeded 
                    saveNeeded = 1
                    print printPrefix+'Added XC10 public certificate into '+keyStoreName+' '+keyStoreScope
                    return
               except JavaException,e2:
                    msg = str(e2)
                    msg.rstrip()
                    msg =  msg[0:len(msg)-1]
                    print printPrefix+'FAILURE: Could not add XC10 public certificate into keystore '+keyStoreName+' '+keyStoreScope+'.'
                    print printPrefix+'         Due to exception: '+msg
                    print printPrefix+'         Please manually add in the public signer certificate: '
                    print printPrefix+'         AdminTask.addSignerCertificate(\'[-certificateAlias '+alias+' -certificateFilePath '+pathToCert+' -keyStoreScope '+keyStoreScope+' -keyStoreName '+keyStoreName+' ]\')'
                    global errors
                    errors = errors + 1
          else:
               print printPrefix+'Unknown error: '+msg
               global errors
               errors = errors + 1

def removeXC10PublicCert(printPrefix, cellName, nodeName):
     try:
          keyStoreScope = '(cell):'+cellName
          keyStoreName = 'CellDefaultTrustStore'
          if (len(nodeName)>0):
               keyStoreName = 'NodeDefaultTrustStore'
               keyStoreScope = keyStoreScope+':(node):'+nodeName

          AdminTask.deleteSignerCertificate('[-keyStoreName '+keyStoreName+' -certificateAlias '+alias+' -keyStoreScope '+keyStoreScope+']')
          global saveNeeded 
          saveNeeded = 1
          print printPrefix+'Removed XC10 public certificate from '+keyStoreName+' '+keyStoreScope
          return 0
     except JavaException, e:
          msg = str(e)
          msg.rstrip()
          msg =  msg[0:len(msg)-1]
          # Exception is thrown if the certificate is not present
          if (msg.find('does not exist') > -1):
               print printPrefix+'XC10 Public certificate does not exist in keystore '+keyStoreName+' '+keyStoreScope+'. Nothing to remove.'
               print printPrefix+'        '+msg
               return 0
          else: 
               print printPrefix+'FAILURE: An exception occurred during the removal of the XC10 public certificate in keystore '+keyStoreName+' '+keyStoreScope+': '+msg
               global errors
               errors = errors + 1
               return -1

#
# Start off main() program
#

print "############################################################"
print "## Add XC10 Public Certificate into all default truststores"
print "## Version:  "+SCRIPT_VERSION
print "############################################################"
print ""

force = 0
remove = 0
was_home = System.getenv("WAS_HOME")
pathToCertificate = was_home+'/properties/xc10public.key'

if (len(sys.argv)>0):
   for arg in range(len(sys.argv)):
       if (sys.argv[arg] == '-force'):
            force = 1
       elif (sys.argv[arg] == '-certPath'):
          arg = arg+1
          pathToCertificate = sys.argv[arg]
       elif (sys.argv[arg] == '-remove'):
          remove = 1

print 'Path to XC10 Public Certificate: '+pathToCertificate
print 'Force XC10 Public Certificate add into TrustStores?: '+str(force)
print 'Remove existing XC10 Public Certificate from all default TrustStores?: '+str(remove)
print ''
print 'Starting XC10 public certificate additions ...'
print ''

# Find the cell name
cellId = AdminConfig.getid("/Cell:/")
cellName = cellId.split("(")[0]

# Add to the cell wide truststore
print 'Processing default truststore for cell '+cellName
printPrefix='     '
if (remove == 0):
     addXC10PublicCert(printPrefix, pathToCertificate,cellName, "", force)
else:
     removeXC10PublicCert(printPrefix, cellName,"")

# Add to each node's truststore
nodes = convertToList(AdminTask.listNodes())
dmgrNode = ""
for node in nodes:
     dmgrId = AdminConfig.getid('/Node:'+node+'/Server:dmgr/')
     if (dmgrId!= None and len(dmgrId)>0 ):
          dmgrNode = node 
nodes.remove(dmgrNode)
print ''
print 'Node List: '+str(nodes)
for node in nodes:
     print 'Processing node '+node+'...'
     if (remove == 0):
          addXC10PublicCert(printPrefix, pathToCertificate, cellName, node, force)
     else:
          removeXC10PublicCert(printPrefix, cellName, node)
     print ''
print 'Completed XC10 public certificate additions.'
if (saveNeeded > 0 and errors == 0):
     print 'Saving configuration changes...'
     AdminConfig.save()
     print 'Configuration saved.'
     print 'Syncing all nodes ... '
     for node in nodes:
          try:
               Sync1 = AdminControl.completeObjectName('type=NodeSync,node='+node+',*')
               if (len(Sync1)>0): 
                    success = AdminControl.invoke(Sync1, 'sync')
                    if (success == 'true'):
                         print '     Node '+node+' successfully synced'
                    else:
                         print '     FAILURE: Node '+node+' sync failed.  Please manually sync node.' 
               else:
                    print '     Node agent for node '+node+' is stopped (can not locate NodSync MBean). Please manually sync node.'
          except JavaException,e:
               msg = str(e)
               print '     FAILURE: Node '+node+' sync failed due to exception.  Please manually sync node. Exception: '+msg 
          
     
     print 'Finished attempting to sync all nodes.'
     print ''
     print 'All WebSphere ND processes that communicate with XC10 appliances, including Deployment Manager, must be restarted to recognize new certificates.'
elif (saveNeeded > 0 and errors > 0):
     print 'Failures experienced during truststore editing, will not save pending changes.'
else:
     print 'No changes were successfully made, skipping configuration save.'